@@ -52,9 +52,9 @@ static retvalue distribution_free(struct distribution *distribution) {
 		free(distribution->version);
 		free(distribution->origin);
 		free(distribution->notautomatic);
+		free(distribution->butautomaticupgrades);
 		free(distribution->label);
 		free(distribution->description);
-		free(distribution->signwith);
 		free(distribution->deb_override);
 		free(distribution->udeb_override);
 		free(distribution->dsc_override);
@@ -62,6 +62,7 @@ static retvalue distribution_free(struct distribution *distribution) {
 		atomlist_done(&distribution->udebcomponents);
 		atomlist_done(&distribution->architectures);
 		atomlist_done(&distribution->components);
+		strlist_done(&distribution->signwith);
 		strlist_done(&distribution->updates);
 		strlist_done(&distribution->pulls);
 		strlist_done(&distribution->alsoaccept);
@@ -387,10 +388,11 @@ CFallSETPROC(distribution, suite)
 CFallSETPROC(distribution, version)
 CFallSETPROC(distribution, origin)
 CFallSETPROC(distribution, notautomatic)
+CFallSETPROC(distribution, butautomaticupgrades)
 CFtruthSETPROC2(distribution, readonly, readonly)
 CFallSETPROC(distribution, label)
 CFallSETPROC(distribution, description)
-CFkeySETPROC(distribution, signwith)
+CFsignwithSETPROC(distribution, signwith)
 CFfileSETPROC(distribution, deb_override)
 CFfileSETPROC(distribution, udeb_override)
 CFfileSETPROC(distribution, dsc_override)
@@ -450,6 +452,7 @@ static const struct configfield distributionconfigfields[] = {
 	CF("Label",		distribution,	label),
 	CF("Log",		distribution,	logger),
 	CF("NotAutomatic",	distribution,	notautomatic),
+	CF("ButAutomaticUpgrades", distribution, butautomaticupgrades),
 	CF("Origin",		distribution,	origin),
 	CF("Pull",		distribution,	pulls),
 	CF("ReadOnly",		distribution,	readonly),
