@@ -35,15 +35,16 @@ struct distribution {
 	 * generated, may be NULL. only suite is sometimes used
 	 * (and only for sanity checks) */
 	/*@null@*/char *suite,*version;
-	/*@null@*/char *origin,*label,*description,*notautomatic;
+	/*@null@*/char *origin, *label, *description,
+		*notautomatic, *butautomaticupgrades;
 	/* What architectures and components are there */
 	struct atomlist architectures, components;
 	/* which update rules to use */
 	struct strlist updates;
 	/* which rules to use to pull packages from other distributions */
 	struct strlist pulls;
-	/* the key to sign with, may be NULL: */
-	/*@null@*/char *signwith;
+	/* the key to sign with, may have no entries to mean unsigned: */
+	struct strlist signwith;
 	/* the override file to use by default */
 	/*@null@*/char *deb_override,*udeb_override,*dsc_override;
 	/* fake component prefix (and codename antisuffix) for Release files: */
