@@ -1803,7 +1803,7 @@ static inline retvalue candidate_checkadd_into(struct database *database,const s
 		return RET_NOTHING;
 }
 
-static inline bool isallowed(UNUSED(struct incoming *i), struct candidate *c, UNUSED(struct distribution *into), struct upload_conditions *conditions) {
+static inline bool isallowed(UNUSED(struct incoming *i), struct candidate *c, struct distribution *into, struct upload_conditions *conditions) {
 	const struct candidate_file *file;
 
 	do switch( uploaders_nextcondition(conditions) ) {
@@ -1811,6 +1811,9 @@ static inline bool isallowed(UNUSED(struct incoming *i), struct candidate *c, UN
 			return true;
 		case uc_REJECTED:
 			return false;
+		case uc_CODENAME:
+			(void)uploaders_verifystring(conditions, into->codename);
+			break;
 		case uc_SOURCENAME:
 			assert( c->source != NULL );
 			(void)uploaders_verifystring(conditions, c->source);
@@ -2021,7 +2024,7 @@ static retvalue candidate_finish_logdir(struct incoming *i, struct candidate *c)
 	return RET_OK;
 }
 
-static retvalue candidate_add_byhands(struct incoming *i, struct candidate *c, struct candidate_perdistribution *d) {
+static retvalue candidate_add_byhands(struct incoming *i, UNUSED(struct candidate *c), struct candidate_perdistribution *d) {
 	struct byhandfile *b;
 	retvalue r;
 
