@@ -1131,6 +1131,8 @@ retvalue release_directorydescription(struct release *release, const struct dist
 	release_writeheader("Architecture",
 			atoms_architectures[target->architecture_atom]);
 	release_writeheader("NotAutomatic",distribution->notautomatic);
+	release_writeheader("ButAutomaticUpgrades",
+			distribution->butautomaticupgrades);
 	release_writeheader("Description",distribution->description);
 #undef release_writeheader
 	r = release_finishfile(release, f);
@@ -1291,6 +1293,10 @@ retvalue release_prepare(struct release *release, struct distribution *distribut
 		writestring("\nNotAutomatic: ");
 		writestring(distribution->notautomatic);
 	}
+	if( distribution->butautomaticupgrades != NULL ) {
+		writestring("\nButAutomaticUpgrades: ");
+		writestring(distribution->butautomaticupgrades);
+	}
 	writechar('\n');
 
 	for( cs = cs_md5sum ; cs < cs_hashCOUNT ; cs++ ) {
@@ -1313,7 +1319,7 @@ retvalue release_prepare(struct release *release, struct distribution *distribut
 			writechar('\n');
 		}
 	}
-	r = signedfile_prepare(release->signedfile, distribution->signwith,
+	r = signedfile_prepare(release->signedfile, &distribution->signwith,
 			!global.keeptemporaries);
 	if( RET_WAS_ERROR(r) ) {
 		signedfile_free(release->signedfile, !global.keeptemporaries);
