@@ -512,6 +512,7 @@ static retvalue parse_changes_files(struct changes *c, struct strlist filelines[
 			p++;
 		if( *p != '\0' ) {
 			fprintf(stderr,"Unexpected sixth argument in '%s'!\n", tmp->values[i]);
+			free(hashes);
 			return RET_ERROR;
 		}
 		if( fileend - filestart == 0 )
@@ -1990,6 +1991,8 @@ static retvalue updatechecksums(const char *changesfilename, struct changes *c,
 			const struct fileentry *sfile = file->dsc->uplink[i];
 			struct checksums **expected_p = &file->dsc->expected.checksums[i];
 			const struct checksums * const expected = *expected_p;
+			const char *hashes1, *hashes2;
+			size_t dummy;
 			bool doit;
 			bool improves;
 
@@ -2032,11 +2035,17 @@ static retvalue updatechecksums(const char *changesfilename, struct changes *c,
 					return r;
 				continue;
 			}
+			r = checksums_getcombined(expected, &hashes1, &dummy);
+			if (!RET_IS_OK(r))
+				hashes1 = "<unknown>";
+			r = checksums_getcombined(sfile->realchecksums,
+					&hashes2, &dummy);
+			if (!RET_IS_OK(r))
+				hashes2 = "<unknown>";
 			fprintf(stderr,
 "Going to update '%s' in '%s'\nfrom '%s'\nto   '%s'.\n",
 					basefilename, file->fullfilename,
-					checksums_getmd5sum(expected),
-					checksums_getmd5sum(sfile->realchecksums));
+					hashes1, hashes2);
 			checksums_free(*expected_p);
 			*expected_p = checksums_dup(sfile->realchecksums);
 			if( *expected_p == NULL )
@@ -2054,6 +2063,8 @@ static retvalue updatechecksums(const char *changesfilename, struct changes *c,
 	memset(improvedfilehashes, 0, sizeof(improvedfilehashes));
 	for( file = c->files; file != NULL ; file = file->next ) {
 		bool improves;
+		const char *hashes1, *hashes2;
+		size_t dummy;
 
 		if( file->checksumsfromchanges == NULL )
 			/* nothing to check here */
@@ -2072,11 +2083,18 @@ static retvalue updatechecksums(const char *changesfilename, struct changes *c,
 				return r;
 			continue;
 		}
+		r = checksums_getcombined(file->checksumsfromchanges,
+				&hashes1, &dummy);
+		if (!RET_IS_OK(r))
+			hashes1 = "<unknown>";
+		r = checksums_getcombined(file->realchecksums,
+				&hashes2, &dummy);
+		if (!RET_IS_OK(r))
+			hashes2 = "<unknown>";
 		fprintf(stderr,
 "Going to update '%s' in '%s'\nfrom '%s'\nto   '%s'.\n",
 				file->basename, changesfilename,
-				checksums_getmd5sum(file->checksumsfromchanges),
-				checksums_getmd5sum(file->realchecksums));
+				hashes1, hashes2);
 		checksums_free(file->checksumsfromchanges);
 		file->checksumsfromchanges = checksums_dup(file->realchecksums);
 		if( file->checksumsfromchanges == NULL )
@@ -2864,12 +2882,13 @@ int main(int argc,char *argv[]) {
 	static const struct option longopts[] = {
 		{"help", no_argument, NULL, 'h'},
 		{"create", no_argument, NULL, 'C'},
-		{"create-with-all-fields", no_argument, &longoption, 5},
+		{"create-with-all-fields", no_argument, &longoption, 6},
 		{"searchpath", required_argument, NULL, 's'},
 		{"gunzip", required_argument, &longoption, 1},
 		{"bunzip2", required_argument, &longoption, 2},
 		{"unlzma", required_argument, &longoption, 3},
 		{"unxz", required_argument, &longoption, 4},
+		{"lunzip", required_argument, &longoption, 5},
 		{NULL, 0, NULL, 0},
 	};
 	int c;
@@ -2879,7 +2898,7 @@ int main(int argc,char *argv[]) {
 	bool all_fields = false;
 	struct strlist searchpath;
 	struct changes *changesdata IFSTUPIDCC(=NULL);
-	char *gunzip = NULL, *bunzip2 = NULL, *unlzma = NULL, *unxz = NULL;
+	char *gunzip = NULL, *bunzip2 = NULL, *unlzma = NULL, *unxz = NULL, *lunzip = NULL;
 	retvalue r;
 
 	strlist_init(&searchpath);
@@ -2901,6 +2920,9 @@ int main(int argc,char *argv[]) {
 						unxz = strdup(optarg);
 						break;
 					case 5:
+						lunzip = strdup(optarg);
+						break;
+					case 6:
 						create_file = true;
 						all_fields = true;
 						break;
@@ -2925,7 +2947,7 @@ int main(int argc,char *argv[]) {
 		about(false);
 	}
 	signature_init(false);
-	uncompressions_check(gunzip, bunzip2, unlzma, unxz);
+	uncompressions_check(gunzip, bunzip2, unlzma, unxz, lunzip);
 
 	changesfilename = argv[optind];
 	if( strcmp(changesfilename,"-") != 0 && !endswith(changesfilename,".changes") ) {
