@@ -39,14 +39,14 @@
 #include "uncompression.h"
 
 const char * const uncompression_suffix[c_COUNT] = {
-	"", ".gz", ".bz2", ".lzma", ".xz" };
+	"", ".gz", ".bz2", ".lzma", ".xz", ".lz" };
 
 /* So help messages can hint what option to try */
 const char * const uncompression_option[c_COUNT] = {
-	NULL, NULL, "--bunzip2", "--unlzma", "--unxz" };
+	NULL, NULL, "--bunzip2", "--unlzma", "--unxz", "--lunzip" };
 /* how those are called in the config file */
 const char * const uncompression_config[c_COUNT] = {
-	".", ".gz", ".bz2", ".lzma", ".xz" };
+	".", ".gz", ".bz2", ".lzma", ".xz", "lz" };
 
 
 /*@null@*/ char *extern_uncompressors[c_COUNT] = {
@@ -340,11 +340,12 @@ static void search_binary(/*@null@*/const char *setting, const char *default_pro
 }
 
 /* check for existance of external programs */
-void uncompressions_check(const char *gunzip, const char *bunzip2, const char *unlzma, const char *unxz) {
+void uncompressions_check(const char *gunzip, const char *bunzip2, const char *unlzma, const char *unxz, const char *lunzip) {
 	search_binary(gunzip,  "gunzip",  &extern_uncompressors[c_gzip]);
 	search_binary(bunzip2, "bunzip2", &extern_uncompressors[c_bzip2]);
 	search_binary(unlzma,  "unlzma",  &extern_uncompressors[c_lzma]);
 	search_binary(unxz,    "unxz",    &extern_uncompressors[c_xz]);
+	search_binary(lunzip,  "lunzip",  &extern_uncompressors[c_lunzip]);
 }
 
 static inline retvalue builtin_uncompress(const char *compressed, const char *destination, enum compression compression) {
@@ -522,7 +523,9 @@ struct compressedfile {
 	} intermediate;
 	union {
 		gzFile gz;
+#ifdef HAVE_LIBBZ2
 		BZFILE *bz;
+#endif
 	};
 };
 
